@@ -271,8 +271,8 @@ static void screen_saver_create_floaters (ScreenSaver *screen_saver);
 static void screen_saver_destroy_floaters (ScreenSaver *screen_saver);
 static void screen_saver_on_size_allocate (ScreenSaver   *screen_saver,
                                            GtkAllocation *allocation);
-static void screen_saver_on_expose_event (ScreenSaver    *screen_saver,
-                                          GdkEventExpose *event);
+static void screen_saver_on_draw_request (ScreenSaver    *screen_saver,
+                                          cairo_t        *context);
 static gboolean do_print_screen_saver_stats (ScreenSaver *screen_saver);
 static GdkPixbuf *gamma_correct (const GdkPixbuf *input_pixbuf);
 
@@ -846,8 +846,8 @@ screen_saver_new (GtkDrawingArea  *drawing_area,
                             G_CALLBACK (screen_saver_on_size_allocate),
                             screen_saver);
 
-  g_signal_connect_swapped (G_OBJECT (drawing_area), "expose-event",
-                            G_CALLBACK (screen_saver_on_expose_event),
+  g_signal_connect_swapped (G_OBJECT (drawing_area), "draw",
+                            G_CALLBACK (screen_saver_on_draw_request),
                             screen_saver);
 
   screen_saver->first_update_time = 0.0;
@@ -997,45 +997,26 @@ compare_floaters (ScreenSaverFloater *a,
 }
 
 static void
-screen_saver_on_expose_event (ScreenSaver    *screen_saver,
-                              GdkEventExpose *event)
+screen_saver_on_draw_request (ScreenSaver    *screen_saver,
+                              cairo_t        *context)
 {
   GList *tmp;
-  cairo_t *context;
 
   if (screen_saver->floaters == NULL)
     screen_saver_create_floaters (screen_saver);
 
-  context = gdk_cairo_create (screen_saver->drawing_area->window);
-
-  cairo_rectangle (context,
-                   (double) event->area.x,
-                   (double) event->area.y,
-                   (double) event->area.width,
-                   (double) event->area.height);
-  cairo_clip (context);
-
   screen_saver->floaters = g_list_sort (screen_saver->floaters,
                                         (GCompareFunc)compare_floaters);
 
   for (tmp = screen_saver->floaters; tmp != NULL; tmp = tmp->next)
     {
       ScreenSaverFloater *floater;
-      GdkRectangle rect;
       gint size;
 
       floater = (ScreenSaverFloater *) tmp->data;
 
       size = CLAMP ((int) (FLOATER_MAX_SIZE * floater->scale),
-		    FLOATER_MIN_SIZE, FLOATER_MAX_SIZE);
-
-      rect.x = (int) (floater->position.x - .5 * G_SQRT2 * size);
-      rect.y = (int) (floater->position.y - .5 * G_SQRT2 * size);
-      rect.width = G_SQRT2 * size;
-      rect.height = G_SQRT2 * size;
-
-      if (!gdk_region_rect_in (event->region, &rect))
-        continue;
+                    FLOATER_MIN_SIZE, FLOATER_MAX_SIZE);
 
       if (!screen_saver_floater_do_draw (screen_saver, floater, context))
         {
@@ -1044,8 +1025,6 @@ screen_saver_on_expose_event (ScreenSaver    *screen_saver,
         }
     }
 
-  cairo_destroy (context);
-
   screen_saver->draw_ops_pending = TRUE;
   screen_saver->frame_count++;
 }
@@ -1064,7 +1043,7 @@ screen_saver_update_state (ScreenSaver *screen_saver,
 
       screen_saver_floater_update_state (screen_saver, floater, time);
 
-      if (GTK_WIDGET_REALIZED (screen_saver->drawing_area)
+      if (gtk_widget_get_realized (screen_saver->drawing_area)
           && (floater->bounds.width > 0) && (floater->bounds.height > 0))
         {
           gint size;
@@ -1180,6 +1159,7 @@ main (int   argc,
   ScreenSaver *screen_saver;
   GtkWidget *window;
   GtkWidget *drawing_area;
+  GtkStyle *style;
 
   GtkStateType state;
 
@@ -1195,7 +1175,7 @@ main (int   argc,
                       /* translators: the word "image" here
                        * represents a command line argument
                        */
-                      _("image - floats images around the screen"),
+                      _("image — floats images around the screen"),
                       options, GETTEXT_PACKAGE, &error);
 
 
@@ -1222,10 +1202,11 @@ main (int   argc,
 
   drawing_area = gtk_drawing_area_new ();
 
+  style = gtk_widget_get_style (drawing_area);
   state = (GtkStateType) 0;
-  while (state < (GtkStateType) G_N_ELEMENTS (drawing_area->style->bg))
+  while (state < (GtkStateType) G_N_ELEMENTS (style->bg))
     {
-      gtk_widget_modify_bg (drawing_area, state, &drawing_area->style->mid[state]);
+      gtk_widget_modify_bg (drawing_area, state, &style->mid[state]);
       state++;
     }
 
