@@ -35,6 +35,8 @@
 static void     gste_popsquares_class_init (GSTEPopsquaresClass *klass);
 static void     gste_popsquares_init       (GSTEPopsquares      *engine);
 static void     gste_popsquares_finalize   (GObject            *object);
+static void     draw_frame                 (GSTEPopsquares *pop,
+                                            cairo_t        *cr);
 
 typedef struct _square {
         int x, y, w, h;
@@ -48,7 +50,6 @@ struct GSTEPopsquaresPrivate
         int        ncolors;
         int        subdivision;
 
-        GdkGC     *gc;
         GdkColor  *colors;
         square    *squares;
 
@@ -182,8 +183,7 @@ rgb_to_hsv (unsigned short r,
 }
 
 static void
-make_color_ramp (GdkColormap *colormap,
-		 int          h1,
+make_color_ramp (int          h1,
                  double       s1,
                  double       v1,
 		 int          h2,
@@ -192,7 +192,6 @@ make_color_ramp (GdkColormap *colormap,
 		 GdkColor    *colors,
                  int          n_colors,
 		 gboolean     closed,
-		 gboolean     allocate,
 		 gboolean     writable)
 {
         double   dh, ds, dv;		/* deltas */
@@ -229,12 +228,6 @@ make_color_ramp (GdkColormap *colormap,
                             &colors [i].red,
                             &colors [i].green,
                             &colors [i].blue);
-                if (allocate) {
-                        gdk_colormap_alloc_color (colormap,
-                                                  &colors [i],
-                                                  writable,
-                                                  TRUE);
-                }
         }
 
         if (closed) {
@@ -270,11 +263,11 @@ set_colors (GdkWindow *window,
 
         widget = gtk_invisible_new ();
 
-        color = widget->style->dark [GTK_STATE_SELECTED];
+        color = gtk_widget_get_style (widget)->dark [GTK_STATE_SELECTED];
         fg->red   = color.red;
         fg->green = color.green;
         fg->blue  = color.blue;
-        color = widget->style->bg [GTK_STATE_SELECTED];
+        color = gtk_widget_get_style (widget)->bg [GTK_STATE_SELECTED];
         bg->red   = color.red;
         bg->green = color.green;
         bg->blue  = color.blue;
@@ -373,11 +366,6 @@ setup_colors (GSTEPopsquares *pop)
 
         set_colors (window, &fg, &bg);
 
-        if (pop->priv->gc) {
-                g_object_unref (pop->priv->gc);
-        }
-        pop->priv->gc = gdk_gc_new (window);
-
         if (pop->priv->colors) {
                 g_free (pop->priv->colors);
         }
@@ -386,13 +374,11 @@ setup_colors (GSTEPopsquares *pop)
         rgb_to_hsv (fg.red, fg.green, fg.blue, &h1, &s1, &v1);
         rgb_to_hsv (bg.red, bg.green, bg.blue, &h2, &s2, &v2);
 
-        make_color_ramp (gtk_widget_get_colormap (GTK_WIDGET (pop)),
-                         h1, s1, v1,
+        make_color_ramp (h1, s1, v1,
                          h2, s2, v2,
                          pop->priv->colors,
                          pop->priv->ncolors,
                          TRUE,
-                         TRUE,
                          FALSE);
 
         nsquares = pop->priv->subdivision * pop->priv->subdivision;
@@ -415,20 +401,16 @@ gste_popsquares_real_show (GtkWidget *widget)
 }
 
 static gboolean
-gste_popsquares_real_expose (GtkWidget      *widget,
-                             GdkEventExpose *event)
+gste_popsquares_real_draw (GtkWidget      *widget,
+                           cairo_t        *cr)
 {
-        gboolean handled = FALSE;
-
-        /* draw */
-
-        /* FIXME: should double buffer? */
-
-        if (GTK_WIDGET_CLASS (parent_class)->expose_event) {
-                handled = GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+        if (GTK_WIDGET_CLASS (parent_class)->draw) {
+                GTK_WIDGET_CLASS (parent_class)->draw (widget, cr);
         }
 
-        return handled;
+        draw_frame (GSTE_POPSQUARES (widget), cr);
+
+        return TRUE;
 }
 
 static gboolean
@@ -467,14 +449,15 @@ gste_popsquares_class_init (GSTEPopsquaresClass *klass)
         object_class->set_property = gste_popsquares_set_property;
 
         widget_class->show = gste_popsquares_real_show;
-        widget_class->expose_event = gste_popsquares_real_expose;
+        widget_class->draw = gste_popsquares_real_draw;
         widget_class->configure_event = gste_popsquares_real_configure;
 
         g_type_class_add_private (klass, sizeof (GSTEPopsquaresPrivate));
 }
 
-static gboolean
-draw_iter (GSTEPopsquares *pop)
+static void
+draw_frame (GSTEPopsquares *pop,
+            cairo_t        *cr)
 {
         int      border = 1;
         gboolean twitch = FALSE;
@@ -488,7 +471,7 @@ draw_iter (GSTEPopsquares *pop)
         window = gs_theme_engine_get_window (GS_THEME_ENGINE (pop));
 
         if (window == NULL) {
-                return TRUE;
+                return;
         }
 
         gs_theme_engine_get_window_size (GS_THEME_ENGINE (pop),
@@ -504,11 +487,11 @@ draw_iter (GSTEPopsquares *pop)
         for (y = 0; y < gh; y++) {
                 for (x = 0; x < gw; x++) {
                         square *s = (square *) &pop->priv->squares [gw * y + x];
-
-                        gdk_gc_set_foreground (pop->priv->gc, &(pop->priv->colors [s->color]));
-                        gdk_draw_rectangle (window, pop->priv->gc, TRUE, s->x, s->y,
-                                            border ? s->w - border : s->w,
-                                            border ? s->h - border : s->h);
+                        gdk_cairo_set_source_color (cr, &(pop->priv->colors [s->color]));
+                        cairo_rectangle (cr, s->x, s->y,
+                                         border ? s->w - border : s->w,
+                                         border ? s->h - border : s->h);
+                        cairo_fill (cr);
                         s->color++;
 
                         if (s->color == pop->priv->ncolors) {
@@ -520,7 +503,12 @@ draw_iter (GSTEPopsquares *pop)
                         }
                 }
         }
+}
 
+static gboolean
+draw_iter (GSTEPopsquares *pop)
+{
+        gtk_widget_queue_draw (GTK_WIDGET (pop));
         return TRUE;
 }
 
@@ -532,7 +520,7 @@ gste_popsquares_init (GSTEPopsquares *pop)
         pop->priv = GSTE_POPSQUARES_GET_PRIVATE (pop);
 
         pop->priv->ncolors = 128;
-        pop->priv->subdivision = 5;;
+        pop->priv->subdivision = 5;
 
         delay = 25;
         pop->priv->timeout_id = g_timeout_add (delay, (GSourceFunc)draw_iter, pop);
@@ -557,7 +545,6 @@ gste_popsquares_finalize (GObject *object)
 
         g_free (pop->priv->squares);
         g_free (pop->priv->colors);
-        g_object_unref (pop->priv->gc);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
