@@ -27,6 +27,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gio/gio.h>
+#include <gdesktop-enums.h>
 
 #include "gs-prefs.h"
 
@@ -38,11 +39,13 @@ static void gs_prefs_finalize   (GObject      *object);
 #define KEY_LOCK_DISABLE          "disable-lock-screen"
 #define KEY_USER_SWITCH_DISABLE   "disable-user-switching"
 
-#define GS_SETTINGS_SCHEMA "org.gnome.screensaver"
+#define GSM_SETTINGS_SCHEMA "org.gnome.desktop.session"
+#define KEY_ACTIVATE_DELAY "idle-delay"
+
+#define GS_SETTINGS_SCHEMA "org.gnome.desktop.screensaver"
 #define KEY_IDLE_ACTIVATION_ENABLED         "idle-activation-enabled"
 #define KEY_LOCK_ENABLED   "lock-enabled"
 #define KEY_MODE           "mode"
-#define KEY_ACTIVATE_DELAY "idle-delay"
 #define KEY_POWER_DELAY    "power-management-delay"
 #define KEY_LOCK_DELAY     "lock-delay"
 #define KEY_CYCLE_DELAY    "cycle-delay"
@@ -60,6 +63,7 @@ static void gs_prefs_finalize   (GObject      *object);
 struct GSPrefsPrivate
 {
         GSettings *settings;
+        GSettings *session_settings;
         GSettings *lockdown;
 };
 
@@ -129,7 +133,7 @@ gs_prefs_class_init (GSPrefsClass *klass)
 
 static void
 _gs_prefs_set_timeout (GSPrefs *prefs,
-                       int      value)
+                       guint    value)
 {
         if (value < 1)
                 value = 10;
@@ -144,7 +148,7 @@ _gs_prefs_set_timeout (GSPrefs *prefs,
 
 static void
 _gs_prefs_set_power_timeout (GSPrefs *prefs,
-                             int      value)
+                             guint    value)
 {
         if (value < 1)
                 value = 60;
@@ -160,11 +164,8 @@ _gs_prefs_set_power_timeout (GSPrefs *prefs,
 
 static void
 _gs_prefs_set_lock_timeout (GSPrefs *prefs,
-                            int      value)
+                            guint    value)
 {
-        if (value < 0)
-                value = 0;
-
         /* pick a reasonable large number for the
            upper bound */
         if (value > 480)
@@ -175,7 +176,7 @@ _gs_prefs_set_lock_timeout (GSPrefs *prefs,
 
 static void
 _gs_prefs_set_cycle_timeout (GSPrefs *prefs,
-                             int      value)
+                             guint    value)
 {
         if (value < 1)
                 value = 1;
@@ -189,8 +190,8 @@ _gs_prefs_set_cycle_timeout (GSPrefs *prefs,
 }
 
 static void
-_gs_prefs_set_mode (GSPrefs    *prefs,
-                    gint        mode)
+_gs_prefs_set_mode (GSPrefs                 *prefs,
+                    GDesktopScreensaverMode  mode)
 {
         prefs->mode = mode;
 }
@@ -291,11 +292,8 @@ _gs_prefs_set_logout_command (GSPrefs    *prefs,
 
 static void
 _gs_prefs_set_logout_timeout (GSPrefs *prefs,
-                              int      value)
+                              guint    value)
 {
-        if (value < 0)
-                value = 0;
-
         /* pick a reasonable large number for the
            upper bound */
         if (value > 480)
@@ -311,14 +309,24 @@ _gs_prefs_set_user_switch_enabled (GSPrefs *prefs,
         prefs->user_switch_enabled = value;
 }
 
+static guint
+_gs_settings_get_uint (GSettings  *settings,
+                       const char *key)
+{
+  guint value;
+
+  g_settings_get (settings, key, "u", &value);
+  return value;
+}
+
 static void
 gs_prefs_load_from_settings (GSPrefs *prefs)
 {
-        glong    value;
+        guint    uvalue;
         gboolean bvalue;
         char    *string;
         gchar  **strv;
-        gint     mode;
+        GDesktopScreensaverMode mode;
 
         bvalue = g_settings_get_boolean (prefs->priv->settings, KEY_IDLE_ACTIVATION_ENABLED);
         _gs_prefs_set_idle_activation_enabled (prefs, bvalue);
@@ -332,17 +340,17 @@ gs_prefs_load_from_settings (GSPrefs *prefs)
         bvalue = g_settings_get_boolean (prefs->priv->lockdown, KEY_USER_SWITCH_DISABLE);
         _gs_prefs_set_user_switch_disabled (prefs, bvalue);
 
-        value = g_settings_get_int (prefs->priv->settings, KEY_ACTIVATE_DELAY);
-        _gs_prefs_set_timeout (prefs, value);
+        uvalue = _gs_settings_get_uint (prefs->priv->session_settings, KEY_ACTIVATE_DELAY);
+        _gs_prefs_set_timeout (prefs, uvalue);
 
-        value = g_settings_get_int (prefs->priv->settings, KEY_POWER_DELAY);
-        _gs_prefs_set_power_timeout (prefs, value);
+        uvalue = _gs_settings_get_uint (prefs->priv->settings, KEY_POWER_DELAY);
+        _gs_prefs_set_power_timeout (prefs, uvalue);
 
-        value = g_settings_get_int (prefs->priv->settings, KEY_LOCK_DELAY);
-        _gs_prefs_set_lock_timeout (prefs, value);
+        uvalue = _gs_settings_get_uint (prefs->priv->settings, KEY_LOCK_DELAY);
+        _gs_prefs_set_lock_timeout (prefs, uvalue);
 
-        value = g_settings_get_int (prefs->priv->settings, KEY_CYCLE_DELAY);
-        _gs_prefs_set_cycle_timeout (prefs, value);
+        uvalue = _gs_settings_get_uint (prefs->priv->settings, KEY_CYCLE_DELAY);
+        _gs_prefs_set_cycle_timeout (prefs, uvalue);
 
         mode = g_settings_get_enum (prefs->priv->settings, KEY_MODE);
         _gs_prefs_set_mode (prefs, mode);
@@ -372,8 +380,8 @@ gs_prefs_load_from_settings (GSPrefs *prefs)
         _gs_prefs_set_logout_command (prefs, string);
         g_free (string);
 
-        value = g_settings_get_int (prefs->priv->settings, KEY_LOGOUT_DELAY);
-        _gs_prefs_set_logout_timeout (prefs, value);
+        uvalue = _gs_settings_get_uint (prefs->priv->settings, KEY_LOGOUT_DELAY);
+        _gs_prefs_set_logout_timeout (prefs, uvalue);
 
         /* User switching options */
 
@@ -383,15 +391,15 @@ gs_prefs_load_from_settings (GSPrefs *prefs)
 
 static void
 key_changed_cb (GSettings   *settings,
-		const gchar *key,
-		GSPrefs     *prefs)
+                const gchar *key,
+                GSPrefs     *prefs)
 {
         if (strcmp (key, KEY_MODE) == 0) {
 
-		gint mode;
+                gint mode;
 
-		mode = g_settings_get_enum (settings, key);
-		_gs_prefs_set_mode (prefs, mode);
+                mode = g_settings_get_enum (settings, key);
+                _gs_prefs_set_mode (prefs, mode);
 
         } else if (strcmp (key, KEY_THEMES) == 0) {
                 gchar **strv = NULL;
@@ -402,108 +410,108 @@ key_changed_cb (GSettings   *settings,
 
         } else if (strcmp (key, KEY_ACTIVATE_DELAY) == 0) {
 
-		int delay;
+                guint delay;
 
-		delay = g_settings_get_int (settings, key);
-		_gs_prefs_set_timeout (prefs, delay);
+                delay = _gs_settings_get_uint (settings, key);
+                _gs_prefs_set_timeout (prefs, delay);
 
         } else if (strcmp (key, KEY_POWER_DELAY) == 0) {
 
-		int delay;
+                guint delay;
 
-		delay = g_settings_get_int (settings, key);
-		_gs_prefs_set_power_timeout (prefs, delay);
+                delay = _gs_settings_get_uint (settings, key);
+                _gs_prefs_set_power_timeout (prefs, delay);
 
         } else if (strcmp (key, KEY_LOCK_DELAY) == 0) {
 
-		int delay;
+                guint delay;
 
-		delay = g_settings_get_int (settings, key);
-		_gs_prefs_set_lock_timeout (prefs, delay);
+                delay = _gs_settings_get_uint (settings, key);
+                _gs_prefs_set_lock_timeout (prefs, delay);
 
         } else if (strcmp (key, KEY_IDLE_ACTIVATION_ENABLED) == 0) {
 
-		gboolean enabled;
+                gboolean enabled;
 
-		enabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_idle_activation_enabled (prefs, enabled);
+                enabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_idle_activation_enabled (prefs, enabled);
 
         } else if (strcmp (key, KEY_LOCK_ENABLED) == 0) {
 
-		gboolean enabled;
+                gboolean enabled;
 
-		enabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_lock_enabled (prefs, enabled);
+                enabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_lock_enabled (prefs, enabled);
 
         } else if (strcmp (key, KEY_LOCK_DISABLE) == 0) {
 
-		gboolean disabled;
+                gboolean disabled;
 
-		disabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_lock_disabled (prefs, disabled);
+                disabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_lock_disabled (prefs, disabled);
 
         } else if (strcmp (key, KEY_USER_SWITCH_DISABLE) == 0) {
 
-		gboolean disabled;
+                gboolean disabled;
 
-		disabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_user_switch_disabled (prefs, disabled);
+                disabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_user_switch_disabled (prefs, disabled);
 
         } else if (strcmp (key, KEY_CYCLE_DELAY) == 0) {
 
-		int delay;
+                guint delay;
 
-		delay = g_settings_get_int (settings, key);
-		_gs_prefs_set_cycle_timeout (prefs, delay);
+                delay = _gs_settings_get_uint (settings, key);
+                _gs_prefs_set_cycle_timeout (prefs, delay);
 
         } else if (strcmp (key, KEY_KEYBOARD_ENABLED) == 0) {
 
-		gboolean enabled;
+                gboolean enabled;
 
-		enabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_keyboard_enabled (prefs, enabled);
+                enabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_keyboard_enabled (prefs, enabled);
 
         } else if (strcmp (key, KEY_KEYBOARD_COMMAND) == 0) {
 
-		const char *command;
+                const char *command;
 
-		command = g_settings_get_string (settings, key);
-		_gs_prefs_set_keyboard_command (prefs, command);
+                command = g_settings_get_string (settings, key);
+                _gs_prefs_set_keyboard_command (prefs, command);
 
         } else if (strcmp (key, KEY_STATUS_MESSAGE_ENABLED) == 0) {
 
-		gboolean enabled;
+                gboolean enabled;
 
-		enabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_status_message_enabled (prefs, enabled);
+                enabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_status_message_enabled (prefs, enabled);
 
         } else if (strcmp (key, KEY_LOGOUT_ENABLED) == 0) {
 
-		gboolean enabled;
+                gboolean enabled;
 
-		enabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_logout_enabled (prefs, enabled);
+                enabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_logout_enabled (prefs, enabled);
 
         } else if (strcmp (key, KEY_LOGOUT_DELAY) == 0) {
 
-		int delay;
+                guint delay;
 
-		delay = g_settings_get_int (settings, key);
-		_gs_prefs_set_logout_timeout (prefs, delay);
+                delay = _gs_settings_get_uint (settings, key);
+                _gs_prefs_set_logout_timeout (prefs, delay);
 
         } else if (strcmp (key, KEY_LOGOUT_COMMAND) == 0) {
 
-		const char *command;
+                const char *command;
 
-		command = g_settings_get_string (settings, key);
-		_gs_prefs_set_logout_command (prefs, command);
+                command = g_settings_get_string (settings, key);
+                _gs_prefs_set_logout_command (prefs, command);
 
         } else if (strcmp (key, KEY_USER_SWITCH_ENABLED) == 0) {
 
-		gboolean enabled;
+                gboolean enabled;
 
-		enabled = g_settings_get_boolean (settings, key);
-		_gs_prefs_set_user_switch_enabled (prefs, enabled);
+                enabled = g_settings_get_boolean (settings, key);
+                _gs_prefs_set_user_switch_enabled (prefs, enabled);
 
         } else {
                 g_warning ("Config key not handled: %s", key);
@@ -518,15 +526,20 @@ gs_prefs_init (GSPrefs *prefs)
         prefs->priv = GS_PREFS_GET_PRIVATE (prefs);
 
         prefs->priv->settings          = g_settings_new (GS_SETTINGS_SCHEMA);
-	g_signal_connect (prefs->priv->settings,
-			  "changed",
-			  G_CALLBACK (key_changed_cb),
-			  prefs);
+        g_signal_connect (prefs->priv->settings,
+                          "changed",
+                          G_CALLBACK (key_changed_cb),
+                          prefs);
+        prefs->priv->session_settings  = g_settings_new (GSM_SETTINGS_SCHEMA);
+        g_signal_connect (prefs->priv->session_settings,
+                          "changed",
+                          G_CALLBACK (key_changed_cb),
+                          prefs);
         prefs->priv->lockdown          = g_settings_new (LOCKDOWN_SETTINGS_SCHEMA);
-	g_signal_connect (prefs->priv->lockdown,
-			  "changed",
-			  G_CALLBACK (key_changed_cb),
-			  prefs);
+        g_signal_connect (prefs->priv->lockdown,
+                          "changed",
+                          G_CALLBACK (key_changed_cb),
+                          prefs);
 
         prefs->idle_activation_enabled = TRUE;
         prefs->lock_enabled            = TRUE;
@@ -561,6 +574,10 @@ gs_prefs_finalize (GObject *object)
                 g_object_unref (prefs->priv->settings);
                 prefs->priv->settings = NULL;
         }
+        if (prefs->priv->session_settings) {
+                g_object_unref (prefs->priv->session_settings);
+                prefs->priv->session_settings = NULL;
+        }
         if (prefs->priv->lockdown) {
                 g_object_unref (prefs->priv->lockdown);
                 prefs->priv->lockdown = NULL;
