@@ -30,7 +30,9 @@
 #include <string.h>
 #include <gdk/gdkx.h>
 
-#include "gs-idle-monitor.h"
+#include <dbus/dbus.h>
+#include <dbus/dbus-glib.h>
+
 #include "gs-watcher.h"
 #include "gs-marshal.h"
 #include "gs-debug.h"
@@ -47,38 +49,31 @@ struct GSWatcherPrivate
 {
         /* settings */
         guint           enabled : 1;
-        guint           timeout;
-        guint           power_timeout;
         guint           delta_notice_timeout;
 
         /* state */
         guint           active : 1;
         guint           idle : 1;
         guint           idle_notice : 1;
-        guint           power_notice : 1;
 
-        GSIdleMonitor  *idle_monitor;
-        guint           power_id;
-        guint           notice_id;
         guint           idle_id;
+        char           *status_message;
 
+        DBusGProxy     *presence_proxy;
         guint           watchdog_timer_id;
+};
 
+enum {
+        PROP_0,
+        PROP_STATUS_MESSAGE
 };
 
 enum {
         IDLE_CHANGED,
         IDLE_NOTICE_CHANGED,
-        POWER_NOTICE_CHANGED,
         LAST_SIGNAL
 };
 
-enum {
-        PROP_0,
-        PROP_TIMEOUT,
-        PROP_POWER_TIMEOUT
-};
-
 static guint signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSWatcher, gs_watcher, G_TYPE_OBJECT)
@@ -101,66 +96,19 @@ add_watchdog_timer (GSWatcher *watcher,
                                                           watcher);
 }
 
-void
-gs_watcher_reset (GSWatcher *watcher)
-{
-        g_return_if_fail (GS_IS_WATCHER (watcher));
-
-        if (watcher->priv->idle_monitor != NULL) {
-                gs_idle_monitor_reset (watcher->priv->idle_monitor);
-        }
-
-        /* restart if necessary */
-        if (watcher->priv->active) {
-                gs_watcher_set_active (watcher, FALSE);
-                gs_watcher_set_active (watcher, TRUE);
-        }
-}
-
-void
-gs_watcher_set_timeout (GSWatcher  *watcher,
-                        guint       timeout)
-{
-        g_return_if_fail (GS_IS_WATCHER (watcher));
-
-        if (watcher->priv->timeout != timeout) {
-                watcher->priv->timeout = timeout;
-
-                /* restart the timers if necessary */
-                gs_watcher_reset (watcher);
-        }
-}
-
-void
-gs_watcher_set_power_timeout (GSWatcher  *watcher,
-                              guint       timeout)
-{
-        g_return_if_fail (GS_IS_WATCHER (watcher));
-
-        if (watcher->priv->power_timeout != timeout) {
-                watcher->priv->power_timeout = timeout;
-
-                /* restart the timers if necessary */
-                gs_watcher_reset (watcher);
-        }
-}
-
 static void
-gs_watcher_set_property (GObject            *object,
-                         guint               prop_id,
-                         const GValue       *value,
-                         GParamSpec         *pspec)
+gs_watcher_get_property (GObject    *object,
+                         guint       prop_id,
+                         GValue     *value,
+                         GParamSpec *pspec)
 {
         GSWatcher *self;
 
         self = GS_WATCHER (object);
 
         switch (prop_id) {
-        case PROP_TIMEOUT:
-                gs_watcher_set_timeout (self, g_value_get_uint (value));
-                break;
-        case PROP_POWER_TIMEOUT:
-                gs_watcher_set_power_timeout (self, g_value_get_uint (value));
+        case PROP_STATUS_MESSAGE:
+                g_value_set_string (value, self->priv->status_message);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -169,21 +117,28 @@ gs_watcher_set_property (GObject            *object,
 }
 
 static void
-gs_watcher_get_property (GObject            *object,
-                         guint               prop_id,
-                         GValue             *value,
-                         GParamSpec         *pspec)
+set_status_text (GSWatcher  *watcher,
+                 const char *text)
+{
+        g_free (watcher->priv->status_message);
+
+        watcher->priv->status_message = g_strdup (text);
+        g_object_notify (G_OBJECT (watcher), "status-message");
+}
+
+static void
+gs_watcher_set_property (GObject          *object,
+                         guint             prop_id,
+                         const GValue     *value,
+                         GParamSpec       *pspec)
 {
         GSWatcher *self;
 
         self = GS_WATCHER (object);
 
         switch (prop_id) {
-        case PROP_TIMEOUT:
-                g_value_set_uint (value, self->priv->timeout);
-                break;
-        case PROP_POWER_TIMEOUT:
-                g_value_set_uint (value, self->priv->power_timeout);
+        case PROP_STATUS_MESSAGE:
+                set_status_text (self, g_value_get_string (value));
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -196,10 +151,18 @@ gs_watcher_class_init (GSWatcherClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->finalize     = gs_watcher_finalize;
+        object_class->finalize = gs_watcher_finalize;
         object_class->get_property = gs_watcher_get_property;
         object_class->set_property = gs_watcher_set_property;
 
+        g_object_class_install_property (object_class,
+                                         PROP_STATUS_MESSAGE,
+                                         g_param_spec_string ("status-message",
+                                                              NULL,
+                                                              NULL,
+                                                              NULL,
+                                                              G_PARAM_READWRITE));
+
         signals [IDLE_CHANGED] =
                 g_signal_new ("idle-changed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -220,62 +183,10 @@ gs_watcher_class_init (GSWatcherClass *klass)
                               gs_marshal_BOOLEAN__BOOLEAN,
                               G_TYPE_BOOLEAN,
                               1, G_TYPE_BOOLEAN);
-        signals [POWER_NOTICE_CHANGED] =
-                g_signal_new ("power-notice-changed",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GSWatcherClass, power_notice_changed),
-                              NULL,
-                              NULL,
-                              gs_marshal_BOOLEAN__BOOLEAN,
-                              G_TYPE_BOOLEAN,
-                              1, G_TYPE_BOOLEAN);
-
-        g_object_class_install_property (object_class,
-                                         PROP_TIMEOUT,
-                                         g_param_spec_uint ("timeout",
-                                                            NULL,
-                                                            NULL,
-                                                            10000,
-                                                            G_MAXUINT,
-                                                            600000,
-                                                            G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-        g_object_class_install_property (object_class,
-                                         PROP_POWER_TIMEOUT,
-                                         g_param_spec_uint ("power-timeout",
-                                                            NULL,
-                                                            NULL,
-                                                            10000,
-                                                            G_MAXUINT,
-                                                            60000,
-                                                            G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
 
         g_type_class_add_private (klass, sizeof (GSWatcherPrivate));
 }
 
-static gboolean
-_gs_watcher_set_session_power_notice (GSWatcher *watcher,
-                                      gboolean   in_effect)
-{
-        gboolean res;
-
-        res = FALSE;
-
-        if (in_effect != watcher->priv->power_notice) {
-
-                g_signal_emit (watcher, signals [POWER_NOTICE_CHANGED], 0, in_effect, &res);
-                if (res) {
-                        gs_debug ("Changing power notice state: %d", in_effect);
-
-                        watcher->priv->power_notice = in_effect;
-                } else {
-                        gs_debug ("Power notice signal not handled: %d", in_effect);
-                }
-        }
-
-        return res;
-}
-
 static gboolean
 _gs_watcher_set_session_idle_notice (GSWatcher *watcher,
                                      gboolean   in_effect)
@@ -322,104 +233,6 @@ _gs_watcher_set_session_idle (GSWatcher *watcher,
         return res;
 }
 
-static gboolean
-on_power_timeout (GSIdleMonitor *monitor,
-                  guint          id,
-                  gboolean       condition,
-                  GSWatcher     *watcher)
-{
-        gboolean res;
-        res = _gs_watcher_set_session_power_notice (watcher, condition);
-        return res;
-}
-
-static gboolean
-on_notice_timeout (GSIdleMonitor *monitor,
-                   guint          id,
-                   gboolean       condition,
-                   GSWatcher     *watcher)
-{
-        gboolean res;
-        res = _gs_watcher_set_session_idle_notice (watcher, condition);
-        return res;
-}
-
-static gboolean
-on_idle_timeout (GSIdleMonitor *monitor,
-                 guint          id,
-                 gboolean       condition,
-                 GSWatcher     *watcher)
-{
-        gboolean res;
-
-        res = _gs_watcher_set_session_idle (watcher, condition);
-        _gs_watcher_set_session_idle_notice (watcher, !condition);
-
-        /* if the event wasn't handled then schedule another timer */
-        if (! res) {
-                gs_debug ("Idle signal was not handled, restarting watcher");
-        }
-
-        return res;
-}
-
-static gboolean
-start_idle_watcher (GSWatcher *watcher)
-{
-        guint notice_timeout;
-
-        g_return_val_if_fail (watcher != NULL, FALSE);
-        g_return_val_if_fail (GS_IS_WATCHER (watcher), FALSE);
-
-        g_debug ("GSWatcher: adding power watch %d", watcher->priv->power_timeout);
-        watcher->priv->power_id
-                = gs_idle_monitor_add_watch (watcher->priv->idle_monitor,
-                                             watcher->priv->power_timeout,
-                                             (GSIdleMonitorWatchFunc)on_power_timeout,
-                                             watcher);
-
-        notice_timeout = watcher->priv->timeout - watcher->priv->delta_notice_timeout;
-        g_debug ("GSWatcher: adding notice watch %d", notice_timeout);
-        watcher->priv->notice_id
-                = gs_idle_monitor_add_watch (watcher->priv->idle_monitor,
-                                             notice_timeout,
-                                             (GSIdleMonitorWatchFunc)on_notice_timeout,
-                                             watcher);
-
-        g_debug ("GSWatcher: adding idle watch %d", watcher->priv->timeout);
-        watcher->priv->idle_id
-                = gs_idle_monitor_add_watch (watcher->priv->idle_monitor,
-                                             watcher->priv->timeout,
-                                             (GSIdleMonitorWatchFunc)on_idle_timeout,
-                                             watcher);
-
-        watchdog_timer (watcher);
-
-        return FALSE;
-}
-
-static gboolean
-stop_idle_watcher (GSWatcher *watcher)
-{
-        g_return_val_if_fail (watcher != NULL, FALSE);
-        g_return_val_if_fail (GS_IS_WATCHER (watcher), FALSE);
-
-        if (watcher->priv->notice_id > 0) {
-                gs_idle_monitor_remove_watch (watcher->priv->idle_monitor,
-                                              watcher->priv->notice_id);
-        }
-        if (watcher->priv->power_id > 0) {
-                gs_idle_monitor_remove_watch (watcher->priv->idle_monitor,
-                                              watcher->priv->power_id);
-        }
-        if (watcher->priv->idle_id > 0) {
-                gs_idle_monitor_remove_watch (watcher->priv->idle_monitor,
-                                              watcher->priv->idle_id);
-        }
-
-        return FALSE;
-}
-
 gboolean
 gs_watcher_get_active (GSWatcher *watcher)
 {
@@ -437,24 +250,17 @@ _gs_watcher_reset_state (GSWatcher *watcher)
 {
         watcher->priv->idle = FALSE;
         watcher->priv->idle_notice = FALSE;
-        watcher->priv->power_notice = FALSE;
 }
 
 static gboolean
 _gs_watcher_set_active_internal (GSWatcher *watcher,
                                  gboolean   active)
 {
-        /* reset state */
-        _gs_watcher_reset_state (watcher);
+        if (active != watcher->priv->active) {
+                /* reset state */
+                _gs_watcher_reset_state (watcher);
 
-        if (! active) {
-                watcher->priv->active = FALSE;
-                stop_idle_watcher (watcher);
-                gs_debug ("Stopping idle watcher");
-        } else {
-                watcher->priv->active = TRUE;
-                start_idle_watcher (watcher);
-                gs_debug ("Starting idle watcher");
+                watcher->priv->active = active;
         }
 
         return TRUE;
@@ -515,6 +321,163 @@ gs_watcher_get_enabled (GSWatcher *watcher)
         return enabled;
 }
 
+static gboolean
+on_idle_timeout (GSWatcher *watcher)
+{
+        gboolean res;
+
+        res = _gs_watcher_set_session_idle (watcher, TRUE);
+
+        _gs_watcher_set_session_idle_notice (watcher, FALSE);
+
+        /* try again if we failed i guess */
+        return !res;
+}
+
+static void
+set_status (GSWatcher *watcher,
+            guint      status)
+{
+        gboolean res;
+        gboolean is_idle;
+
+        if (! watcher->priv->active) {
+                gs_debug ("GSWatcher: not active, ignoring status changes");
+                return;
+        }
+
+        is_idle = (status == 3);
+
+        if (!is_idle && !watcher->priv->idle_notice) {
+                /* no change in idleness */
+                return;
+        }
+
+        if (is_idle) {
+                res = _gs_watcher_set_session_idle_notice (watcher, is_idle);
+                /* queue an activation */
+                if (watcher->priv->idle_id > 0) {
+                        g_source_remove (watcher->priv->idle_id);
+                }
+                watcher->priv->idle_id = g_timeout_add (watcher->priv->delta_notice_timeout,
+                                                        (GSourceFunc)on_idle_timeout,
+                                                        watcher);
+        } else {
+                /* cancel notice too */
+                if (watcher->priv->idle_id > 0) {
+                        g_source_remove (watcher->priv->idle_id);
+                }
+                res = _gs_watcher_set_session_idle (watcher, FALSE);
+                res = _gs_watcher_set_session_idle_notice (watcher, FALSE);
+        }
+}
+
+static void
+on_presence_status_changed (DBusGProxy    *presence_proxy,
+                            guint          status,
+                            GSWatcher     *watcher)
+{
+        set_status (watcher, status);
+}
+
+static void
+on_presence_status_text_changed (DBusGProxy    *presence_proxy,
+                                 const char    *status_text,
+                                 GSWatcher     *watcher)
+{
+        set_status_text (watcher, status_text);
+}
+
+static void
+connect_presence_watcher (GSWatcher *watcher)
+{
+        DBusGConnection *bus;
+        GError          *error;
+        DBusGProxy      *proxy;
+        guint            status;
+        const char      *status_text;
+        GValue           value = { 0, };
+
+        error = NULL;
+        bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+        if (bus == NULL) {
+                g_warning ("Unable to get session bus: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        watcher->priv->presence_proxy = dbus_g_proxy_new_for_name (bus,
+                                                                   "org.gnome.SessionManager",
+                                                                   "/org/gnome/SessionManager/Presence",
+                                                                   "org.gnome.SessionManager.Presence");
+
+        dbus_g_proxy_add_signal (watcher->priv->presence_proxy,
+                                 "StatusChanged",
+                                 G_TYPE_UINT,
+                                 G_TYPE_INVALID);
+        dbus_g_proxy_connect_signal (watcher->priv->presence_proxy,
+                                     "StatusChanged",
+                                     G_CALLBACK (on_presence_status_changed),
+                                     watcher,
+                                     NULL);
+        dbus_g_proxy_add_signal (watcher->priv->presence_proxy,
+                                 "StatusTextChanged",
+                                 G_TYPE_STRING,
+                                 G_TYPE_INVALID);
+        dbus_g_proxy_connect_signal (watcher->priv->presence_proxy,
+                                     "StatusTextChanged",
+                                     G_CALLBACK (on_presence_status_text_changed),
+                                     watcher,
+                                     NULL);
+
+        proxy = dbus_g_proxy_new_from_proxy (watcher->priv->presence_proxy,
+                                             "org.freedesktop.DBus.Properties",
+                                             "/org/gnome/SessionManager/Presence");
+
+        status = 0;
+        status_text = NULL;
+
+        error = NULL;
+        dbus_g_proxy_call (proxy,
+                           "Get",
+                           &error,
+                           G_TYPE_STRING, "org.gnome.SessionManager.Presence",
+                           G_TYPE_STRING, "status",
+                           G_TYPE_INVALID,
+                           G_TYPE_VALUE, &value,
+                           G_TYPE_INVALID);
+
+        if (error != NULL) {
+                g_warning ("Couldn't get presence status: %s", error->message);
+                g_error_free (error);
+                return;
+        } else {
+                status = g_value_get_uint (&value);
+        }
+
+        g_value_unset (&value);
+
+        error = NULL;
+        dbus_g_proxy_call (proxy,
+                           "Get",
+                           &error,
+                           G_TYPE_STRING, "org.gnome.SessionManager.Presence",
+                           G_TYPE_STRING, "status-text",
+                           G_TYPE_INVALID,
+                           G_TYPE_VALUE, &value,
+                           G_TYPE_INVALID);
+
+        if (error != NULL) {
+                g_warning ("Couldn't get presence status text: %s", error->message);
+                g_error_free (error);
+        } else {
+                status_text = g_value_get_string (&value);
+        }
+
+        set_status (watcher, status);
+        set_status_text (watcher, status_text);
+}
+
 static void
 gs_watcher_init (GSWatcher *watcher)
 {
@@ -522,9 +485,8 @@ gs_watcher_init (GSWatcher *watcher)
 
         watcher->priv->enabled = TRUE;
         watcher->priv->active = FALSE;
-        watcher->priv->timeout = 600000;
 
-        watcher->priv->idle_monitor = gs_idle_monitor_new ();
+        connect_presence_watcher (watcher);
 
         /* time before idle signal to send notice signal */
         watcher->priv->delta_notice_timeout = 10000;
@@ -546,8 +508,17 @@ gs_watcher_finalize (GObject *object)
 
         remove_watchdog_timer (watcher);
 
+        if (watcher->priv->idle_id > 0) {
+                g_source_remove (watcher->priv->idle_id);
+        }
+
         watcher->priv->active = FALSE;
-        stop_idle_watcher (watcher);
+
+        if (watcher->priv->presence_proxy != NULL) {
+                g_object_unref (watcher->priv->presence_proxy);
+        }
+
+        g_free (watcher->priv->status_message);
 
         G_OBJECT_CLASS (gs_watcher_parent_class)->finalize (object);
 }
@@ -564,7 +535,7 @@ disable_builtin_screensaver (GSWatcher *watcher,
         int desired_server_timeout, desired_server_interval;
         int desired_prefer_blank,   desired_allow_exp;
 
-        XGetScreenSaver (GDK_DISPLAY (),
+        XGetScreenSaver (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                          &current_server_timeout,
                          &current_server_interval,
                          &current_prefer_blank,
@@ -602,18 +573,18 @@ disable_builtin_screensaver (GSWatcher *watcher,
                           (desired_prefer_blank ? "blank" : "noblank"),
                           (desired_allow_exp ? "expose" : "noexpose"));
 
-                XSetScreenSaver (GDK_DISPLAY (),
+                XSetScreenSaver (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                  desired_server_timeout,
                                  desired_server_interval,
                                  desired_prefer_blank,
                                  desired_allow_exp);
 
-                XSync (GDK_DISPLAY (), FALSE);
+                XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
         }
 
         if (unblank_screen) {
                 /* Turn off the server builtin saver if it is now running. */
-                XForceScreenSaver (GDK_DISPLAY (), ScreenSaverReset);
+                XForceScreenSaver (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), ScreenSaverReset);
         }
 }
 
@@ -640,12 +611,11 @@ watchdog_timer (GSWatcher *watcher)
 }
 
 GSWatcher *
-gs_watcher_new (guint timeout)
+gs_watcher_new (void)
 {
         GSWatcher *watcher;
 
         watcher = g_object_new (GS_TYPE_WATCHER,
-                                "timeout", timeout,
                                 NULL);
 
         return GS_WATCHER (watcher);
